% noisemap_trick.m
%
% David Hale, CIT <dhale@astro.caltech.edu>
%
% This script computes the noise vs. sampling freq for a "filmstrip"
% light image.

beep on

display('noisemap_trick, 2013-08-19')

indirok=0;  % set TRUE after a valid directory is entered
while indirok==0,
    indirname = input('Enter data directory name: ', 's');
    if isempty(indirname)
        indirok=0;
    else
        indirok=1;
    end
end % input directory name

% make sure there is a trailing backslash
s=size(indirname);
s=s(2);
if indirname(s) ~= '/'
    indirname=strcat(indirname,'/');
end

readme = input('alternate README file? (<ENTER> for README.grid in data direcctory: ', 's');
if isempty(readme)
    readme=strcat(indirname,'README.grid');
end

outdirname = input('Enter output directory name or <ENTER> for /working under data directory: ', 's');
if isempty(outdirname)
    outdirname = strcat(indirname,'working/');
end
% make sure there is a trailing backslash
s=size(outdirname);
s=s(2);
if outdirname(s) ~= '/'
    outdirname=strcat(outdirname,'/');
end

[filename,start_row,start_col]=textread(readme,'%s%d%d%*[^\n]','delimiter',' ','headerlines',1);

numfiles=size(filename);
numfiles=numfiles(1);

% enter conversion gain, if known
G=input('Enter conversion gain, if known (<ENTER> for 0.9): ', 's');
if ~isempty(G)
    G=str2double(G);
else
    G=0.9;
end

% input max fowler depth (or use predefined set for making plots)
finput=input('Enter number of Fowler samples (<ENTER> for defaults): ');
if (isempty(finput))
    Fnset  = [ 1 2 4 8 16 27 81 162 406 811 ];
else
    Fnset = [finput];
end

% period to save data durring analysis
saveperiod=input('Periodically write data files (input period or <ENTER> for none): ');
if (isempty(saveperiod))
    saveperiod=0;
end

Fnsize = size(Fnset);
numFns = Fnsize(2);

% output FITS filenames for noisemaps, one for each frequency
noisebasename=strcat(outdirname,'noisemap_');
imagebasename=strcat(outdirname,'image_');
noiseoutfile=cell(numFns,1);
imageoutfile=cell(numFns,1);
for i=1:numFns
    noiseoutfile{i}=strcat(noisebasename,int2str(i));
    noiseoutfile{i}=strcat(noiseoutfile{i},'.fits');
    imageoutfile{i}=strcat(imagebasename,int2str(i));
    imageoutfile{i}=strcat(imageoutfile{i},'.fits');
end
savefile=strcat(outdirname,'noisemap_data.mat');

maxdepth = max(Fnset);

% this script assumes a fixed window size for all files in a single directory
rows=4; cols=rows;
totgridcols=512;
totgridrows=512;

% one cell element for each Fowler depth
% only define this if not defined, which allows re-running the script
% to build up the structure in stages
if ~exist('noisedata')
    array = zeros(totgridrows*rows, totgridcols*cols);
    noisedata = cell(1,numFns);
    imagedata = cell(1,numFns);
    [noisedata{:}] = deal(array);
    [imagedata{:}] = deal(array);
    rms_noise = zeros(rows, cols, maxdepth);
end

x=1;
y=1;

filenum=1;

while filenum <= (numfiles),
    
    inputfile=strcat(indirname,char(filename(filenum)));
    display('  reading input FITS file...');
    im=fitsread(inputfile);
    FZ=im(1:4,1:4);

    first_good_row=1;
    lincol    = size(im);
    lines     = lincol(1);
    true_rows = lincol(2);      % the actual number of rows (not subarray)
    frames    = lines / true_rows;  % again, assumes square readout region

    % sometimes TRICK filmstrips read 0. Find the first non-zero frame in the filmstrip.
    for i=first_good_row:rows:frames
        if mean(mean(im(i:i+rows-1,1:cols)))~=0
            break
        end
    end

    % recalculate lines and frames
    first_good_row=i;
    lines=lines-first_good_row+1;
    frames=lines/true_rows;

    fprintf('  computing Fowler images for file %03d/%03d (%s)  row=%4d col=%4d...\n', filenum, numfiles, char(filename(filenum)), start_row(filenum), start_col(filenum));

    % create arrays ahead of time...
    noise_surf   = zeros(10, 2, numFns);

    Fn = 1;  % the first Fowler number
    k  = 1;

    while (k) <= numFns,
        Fn = Fnset(k);                              % the current requested Fowler number
        Ns = floor(frames/Fn);                      % number of samples possible at this Fn
    
        fprintf('  computing Fowler depth %d with %d samples (%.1f Hz)...\n ', Fn, Ns, 1./(Fn*0.1233e-3));
        fowler_avg = zeros(rows, cols, Ns);         % store Ns fowler frame averages
        fowler_sub = zeros(rows, cols, Ns-1);       % store Ns-1 fowler subtraction frames

        i = first_good_row;
        for z = 1:Ns
            fowler_sum = zeros(rows,cols);          % zero-out temporary summation matrix
            for j = 1:Fn
              temp = G * im(i:i+rows-1, 1:cols);    % apply gain
              fowler_sum(:,:) = fowler_sum + temp;  % store sum in here
              i = i + true_rows;                    % skip to start of next frame
            end
            fowler_avg(:,:,z) = fowler_sum./Fn;     % divide by number of sums
            
            if (z>1)
                fowler_sub(:,:,z-1) = fowler_avg(:,:,z) - fowler_avg(:,:,z-1);
            end
        end

        rms_noise(:,:,Fn) = std(fowler_sub,0,3);
        mean_image(:,:,Fn) = mean(fowler_sub,3);
    
        k=k+1;
    end

    
    % store the calculated noise in a cell
    % the cell elements are for the different sampling frequencies
    for j=1:numFns
        [noisedata{1,j}(start_row(filenum)+1:start_row(filenum)+rows, start_col(filenum)+1:start_col(filenum)+cols)] = rms_noise(:,:,Fnset(j));
        [imagedata{1,j}(start_row(filenum)+1:start_row(filenum)+rows, start_col(filenum)+1:start_col(filenum)+cols)] = mean_image(:,:,Fnset(j));
    end
        
    % periodicly save data
    if (mod(filenum,saveperiod)==0)
        fprintf('  periodic write FITS output files\n');
        for j=1:numFns
            fitswrite(((noisedata{1,j})), char(noiseoutfile{j}));
            fitswrite(((imagedata{1,j})), char(imageoutfile{j}));
        end
        fprintf('  periodic save Matlab variable\n');
        save(savefile, 'noisedata', 'imagedata');
    end
        
    filenum=filenum+1;
end

% one final write (in case periodic save not integer multiple)
fprintf('  writing final FITS output files\n');
for j=1:numFns
    fitswrite(((noisedata{1,j})), char(noiseoutfile{j}));
    fitswrite(((imagedata{1,j})), char(imageoutfile{j}));
end
fprintf('  saving final Matlab variable\n');
save(savefile, 'noisedata', 'imagedata');

beep
beep
           
display('  done!');
